@@ -34,15 +34,31 @@
 #include <time.h>
 #include <unistd.h>
 
-#include <glib/gi18n-lib.h>
-#include <gtk/gtkicontheme.h>
+#include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gdk-pixbuf/gdk-pixbuf-loader.h>
 
 #define GWEATHER_I_KNOW_THIS_IS_UNSTABLE
 #include "weather.h"
 #include "weather-priv.h"
 
+static void _weather_internal_check (void);
+
+const char *
+gweather_gettext (const char *str)
+{
+    static gsize gweather_gettext_initialized = FALSE;
+
+    if (G_UNLIKELY (g_once_init_enter (&gweather_gettext_initialized))) {
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#endif
+	g_once_init_leave (&gweather_gettext_initialized, TRUE);
+    }
+
+    return dgettext (GETTEXT_PACKAGE, str);
+}
+
 /*
  * Convert string of the form "DD-MM-SSH" to radians
  * DD:degrees (to 3 digits), MM:minutes, SS:seconds H:hemisphere (NESW)
@@ -90,6 +106,8 @@ weather_location_new (const gchar *name, const gchar *code,
 {
     WeatherLocation *location;
 
+    _weather_internal_check ();
+
     location = g_new (WeatherLocation, 1);
 
     /* name and metar code must be set */
@@ -151,6 +169,8 @@ weather_location_clone (const WeatherLocation *location)
 {
     WeatherLocation *clone;
 
+    g_return_val_if_fail (location != NULL, NULL);
+
     clone = weather_location_new (location->name,
 				  location->code, location->zone,
 				  location->radar, location->coordinates,
@@ -180,8 +200,14 @@ weather_location_free (WeatherLocation *location)
 gboolean
 weather_location_equal (const WeatherLocation *location1, const WeatherLocation *location2)
 {
+    /* if something is NULL, then it's TRUE if and only if both are NULL) */
+    if (location1 == NULL || location2 == NULL)
+        return (location1 == location2);
     if (!location1->code || !location2->code)
-        return 1;
+        return (location1->code == location2->code);
+    if (!location1->name || !location2->name)
+        return (location1->name == location2->name);
+
     return ((strcmp (location1->code, location2->code) == 0) &&
 	    (strcmp (location1->name, location2->name) == 0));
 }
@@ -197,9 +223,7 @@ static const gchar *wind_direction_str[] = {
 const gchar *
 weather_wind_direction_string (WeatherWindDirection wind)
 {
-    if (wind < 0)
-	return _("Unknown");
-    if (wind >= (sizeof (wind_direction_str) / sizeof (char *)))
+    if (wind <= WIND_INVALID || wind >= WIND_LAST)
 	return _("Invalid");
 
     return _(wind_direction_str[(int)wind]);
@@ -216,8 +240,7 @@ static const gchar *sky_str[] = {
 const gchar *
 weather_sky_string (WeatherSky sky)
 {
-    if (sky < 0 ||
-	sky >= (sizeof (sky_str) / sizeof (char *)))
+    if (sky <= SKY_INVALID || sky >= SKY_LAST)
 	return _("Invalid");
 
     return _(sky_str[(int)sky]);
@@ -284,10 +307,10 @@ weather_conditions_string (WeatherConditions cond)
     if (!cond.significant) {
 	return "-";
     } else {
-	if (cond.phenomenon >= 0 &&
-	    cond.phenomenon < 24 &&
-	    cond.qualifier >= 0 &&
-	    cond.qualifier < 13)
+	if (cond.phenomenon > PHENOMENON_INVALID &&
+	    cond.phenomenon < PHENOMENON_LAST &&
+	    cond.qualifier > QUALIFIER_INVALID &&
+	    cond.qualifier < QUALIFIER_LAST)
 	    str = _(conditions_str[(int)cond.phenomenon][(int)cond.qualifier]);
 	else
 	    str = _("Invalid");
@@ -316,8 +339,25 @@ void request_done (WeatherInfo *info, gboolean ok)
         info->finish_cb (info, info->cb_data);
 }
 
-/* Relative humidity computation - thanks to <Olof.Oberg@modopaper.modogroup.com> */
+/* it's OK to pass in NULL */
+void
+free_forecast_list (WeatherInfo *info)
+{
+    GSList *p;
 
+    if (!info)
+	return;
+
+    for (p = info->forecast_list; p; p = p->next)
+	weather_info_free (p->data);
+
+    if (info->forecast_list) {
+	g_slist_free (info->forecast_list);
+	info->forecast_list = NULL;
+    }
+}
+
+/* Relative humidity computation - thanks to <Olof.Oberg@modopaper.modogroup.com> */
 
 static inline gdouble
 calc_humidity (gdouble temp, gdouble dewp)
@@ -416,7 +456,6 @@ calc_apparent (WeatherInfo *info)
     return apparent;
 }
 
-
 WeatherInfo *
 _weather_info_fill (WeatherInfo *info,
 		    WeatherLocation *location,
@@ -437,8 +476,10 @@ _weather_info_fill (WeatherInfo *info,
         location = info->location;
 	if (info->forecast)
 	    g_free (info->forecast);
-
 	info->forecast = NULL;
+
+	free_forecast_list (info);
+
 	if (info->radar != NULL) {
 	    g_object_unref (info->radar);
 	    info->radar = NULL;
@@ -466,6 +507,9 @@ _weather_info_fill (WeatherInfo *info,
     info->cond.phenomenon = PHENOMENON_NONE;
     info->cond.qualifier = QUALIFIER_NONE;
     info->temp = -1000.0;
+    info->tempMinMaxValid = FALSE;
+    info->temp_min = -1000.0;
+    info->temp_max = -1000.0;
     info->dew = -1000.0;
     info->wind = -1;
     info->windspeed = -1;
@@ -475,14 +519,19 @@ _weather_info_fill (WeatherInfo *info,
     info->sunrise = 0;
     info->sunset = 0;
     info->forecast = NULL;
+    info->forecast_list = NULL;
     info->radar = NULL;
     info->radar_url = prefs->radar && prefs->radar_custom_url ?
     		      g_strdup (prefs->radar_custom_url) : NULL;
     info->finish_cb = cb;
     info->cb_data = data;
 
-    if (!info->session)
+    if (!info->session) {
 	info->session = soup_session_async_new ();
+#ifdef HAVE_LIBSOUP_GNOME
+	soup_session_add_feature_by_type (info->session, SOUP_TYPE_PROXY_RESOLVER_GNOME);
+#endif
+    }
 
     metar_start_open (info);
     iwin_start_open (info);
@@ -497,6 +546,8 @@ _weather_info_fill (WeatherInfo *info,
 void
 weather_info_abort (WeatherInfo *info)
 {
+    g_return_if_fail (info != NULL);
+
     if (info->session) {
 	soup_session_abort (info->session);
 	info->requests_pending = 0;
@@ -523,6 +574,17 @@ weather_info_clone (const WeatherInfo *info)
     clone->forecast = g_strdup (info->forecast);
     clone->radar_url = g_strdup (info->radar_url);
 
+    if (info->forecast_list) {
+	GSList *p;
+
+	clone->forecast_list = NULL;
+	for (p = info->forecast_list; p; p = p->next) {
+	    clone->forecast_list = g_slist_prepend (clone->forecast_list, weather_info_clone (p->data));
+	}
+
+	clone->forecast_list = g_slist_reverse (clone->forecast_list);
+    }
+
     clone->radar = info->radar;
     if (clone->radar != NULL)
 	g_object_ref (clone->radar);
@@ -546,6 +608,8 @@ weather_info_free (WeatherInfo *info)
     g_free (info->forecast);
     info->forecast = NULL;
 
+    free_forecast_list (info);
+
     if (info->radar != NULL) {
         g_object_unref (info->radar);
         info->radar = NULL;
@@ -561,6 +625,35 @@ weather_info_is_valid (WeatherInfo *info)
     return info->valid;
 }
 
+gboolean
+weather_info_network_error (WeatherInfo *info)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    return info->network_error;
+}
+
+void
+weather_info_to_metric (WeatherInfo *info)
+{
+    g_return_if_fail (info != NULL);
+
+    info->temperature_unit = TEMP_UNIT_CENTIGRADE;
+    info->speed_unit = SPEED_UNIT_MS;
+    info->pressure_unit = PRESSURE_UNIT_HPA;
+    info->distance_unit = DISTANCE_UNIT_METERS;
+}
+
+void
+weather_info_to_imperial (WeatherInfo *info)
+{
+    g_return_if_fail (info != NULL);
+
+    info->temperature_unit = TEMP_UNIT_FAHRENHEIT;
+    info->speed_unit = SPEED_UNIT_MPH;
+    info->pressure_unit = PRESSURE_UNIT_INCH_HG;
+    info->distance_unit = DISTANCE_UNIT_MILES;
+}
+
 const WeatherLocation *
 weather_info_get_location (WeatherInfo *info)
 {
@@ -643,20 +736,20 @@ temperature_string (gfloat far, TempUnit to_unit, gboolean round)
     switch (to_unit) {
     case TEMP_UNIT_FAHRENHEIT:
 	if (!round) {
-	    /* TRANSLATOR: This is the temperature in degrees Fahrenheit (\342\204\211 is the "DEGREE FAHRENHEIT" symbol) */
-	    g_snprintf (buf, sizeof (buf), _("%.1f \342\204\211"), far);
+	    /* TRANSLATOR: This is the temperature in degrees Fahrenheit (\302\260 is U+00B0 DEGREE SIGN) */
+	    g_snprintf (buf, sizeof (buf), _("%.1f \302\260F"), far);
 	} else {
-	    /* TRANSLATOR: This is the temperature in degrees Fahrenheit (\342\204\211 is the "DEGREE FAHRENHEIT" symbol) */
-	    g_snprintf (buf, sizeof (buf), _("%d \342\204\211"), (int)floor (far + 0.5));
+	    /* TRANSLATOR: This is the temperature in degrees Fahrenheit (\302\260 is U+00B0 DEGREE SIGN) */
+	    g_snprintf (buf, sizeof (buf), _("%d \302\260F"), (int)floor (far + 0.5));
 	}
 	break;
     case TEMP_UNIT_CENTIGRADE:
 	if (!round) {
-	    /* TRANSLATOR: This is the temperature in degrees Celsius (\342\204\203 is the "DEGREE CELSIUS" symbol) */
-	    g_snprintf (buf, sizeof (buf), _("%.1f \342\204\203"), TEMP_F_TO_C (far));
+	    /* TRANSLATOR: This is the temperature in degrees Celsius (\302\260 is U+00B0 DEGREE SIGN) */
+	    g_snprintf (buf, sizeof (buf), _("%.1f \302\260C"), TEMP_F_TO_C (far));
 	} else {
-	    /* TRANSLATOR: This is the temperature in degrees Celsius (\342\204\203 is the "DEGREE CELSIUS" symbol) */
-	    g_snprintf (buf, sizeof (buf), _("%d \342\204\203"), (int)floor (TEMP_F_TO_C (far) + 0.5));
+	    /* TRANSLATOR: This is the temperature in degrees Celsius (\302\260 is U+00B0 DEGREE SIGN) */
+	    g_snprintf (buf, sizeof (buf), _("%d \302\260C"), (int)floor (TEMP_F_TO_C (far) + 0.5));
 	}
 	break;
     case TEMP_UNIT_KELVIN:
@@ -692,6 +785,32 @@ weather_info_get_temp (WeatherInfo *info)
     return temperature_string (info->temp, info->temperature_unit, FALSE);
 }
 
+const gchar *
+weather_info_get_temp_min (WeatherInfo *info)
+{
+    g_return_val_if_fail (info != NULL, NULL);
+
+    if (!info->valid || !info->tempMinMaxValid)
+        return "-";
+    if (info->temp_min < -500.0)
+        return _("Unknown");
+
+    return temperature_string (info->temp_min, info->temperature_unit, FALSE);
+}
+
+const gchar *
+weather_info_get_temp_max (WeatherInfo *info)
+{
+    g_return_val_if_fail (info != NULL, NULL);
+
+    if (!info->valid || !info->tempMinMaxValid)
+        return "-";
+    if (info->temp_max < -500.0)
+        return _("Unknown");
+
+    return temperature_string (info->temp_max, info->temperature_unit, FALSE);
+}
+
 const gchar *
 weather_info_get_dew (WeatherInfo *info)
 {
@@ -936,6 +1055,26 @@ weather_info_get_forecast (WeatherInfo *info)
     return info->forecast;
 }
 
+/**
+ * weather_info_get_forecast_list:
+ * Returns list of WeatherInfo* objects for the forecast.
+ * The list is owned by the 'info' object thus is alive as long
+ * as the 'info'. This list is filled only when requested with
+ * type FORECAST_LIST and if available for given location.
+ * The 'update' property is the date/time when the forecast info
+ * is used for.
+ **/
+GSList *
+weather_info_get_forecast_list (WeatherInfo *info)
+{
+    g_return_val_if_fail (info != NULL, NULL);
+
+    if (!info->valid)
+	return NULL;
+
+    return info->forecast_list;
+}
+
 GdkPixbufAnimation *
 weather_info_get_radar (WeatherInfo *info)
 {
@@ -946,8 +1085,8 @@ weather_info_get_radar (WeatherInfo *info)
 const gchar *
 weather_info_get_temp_summary (WeatherInfo *info)
 {
-    if (!info)
-        return NULL;
+    g_return_val_if_fail (info != NULL, NULL);
+
     if (!info->valid || info->temp < -500.0)
         return "--";
 
@@ -978,7 +1117,9 @@ weather_info_get_icon_name (WeatherInfo *info)
     time_t current_time;
     gboolean daytime;
 
-    if (!info || !info->valid)
+    g_return_val_if_fail (info != NULL, NULL);
+
+    if (!info->valid)
         return NULL;
 
     cond = info->cond;
@@ -990,6 +1131,8 @@ weather_info_get_icon_name (WeatherInfo *info)
             return "weather-storm";
 
         switch (cond.phenomenon) {
+	case PHENOMENON_INVALID:
+	case PHENOMENON_LAST:
 	case PHENOMENON_NONE:
 	    break;
 
@@ -1033,6 +1176,7 @@ weather_info_get_icon_name (WeatherInfo *info)
 
     switch (sky) {
     case SKY_INVALID:
+    case SKY_LAST:
     case SKY_CLEAR:
 	if (daytime)
 	    return "weather-clear";
@@ -1053,3 +1197,347 @@ weather_info_get_icon_name (WeatherInfo *info)
 
     return NULL;
 }
+
+static gboolean
+temperature_value (gdouble far, TempUnit to_unit, gdouble *value, TempUnit def_unit)
+{
+    gboolean ok = TRUE;
+
+    *value = 0.0;
+    if (far < -500.0)
+	return FALSE;
+
+    if (to_unit == TEMP_UNIT_DEFAULT)
+	    to_unit = def_unit;
+
+    switch (to_unit) {
+        case TEMP_UNIT_FAHRENHEIT:
+	    *value = far;
+	    break;
+        case TEMP_UNIT_CENTIGRADE:
+	    *value = TEMP_F_TO_C (far);
+	    break;
+        case TEMP_UNIT_KELVIN:
+	    *value = TEMP_F_TO_K (far);
+	    break;
+        case TEMP_UNIT_INVALID:
+        case TEMP_UNIT_DEFAULT:
+	default:
+	    ok = FALSE;
+	    break;
+    }
+
+    return ok;
+}
+
+static gboolean
+speed_value (gdouble knots, SpeedUnit to_unit, gdouble *value, SpeedUnit def_unit)
+{
+    gboolean ok = TRUE;
+
+    *value = -1.0;
+
+    if (knots < 0.0)
+	return FALSE;
+
+    if (to_unit == SPEED_UNIT_DEFAULT)
+	    to_unit = def_unit;
+
+    switch (to_unit) {
+        case SPEED_UNIT_KNOTS:
+            *value = knots;
+	    break;
+        case SPEED_UNIT_MPH:
+            *value = WINDSPEED_KNOTS_TO_MPH (knots);
+	    break;
+        case SPEED_UNIT_KPH:
+            *value = WINDSPEED_KNOTS_TO_KPH (knots);
+	    break;
+        case SPEED_UNIT_MS:
+            *value = WINDSPEED_KNOTS_TO_MS (knots);
+	    break;
+	case SPEED_UNIT_BFT:
+	    *value = WINDSPEED_KNOTS_TO_BFT (knots);
+	    break;
+        case SPEED_UNIT_INVALID:
+        case SPEED_UNIT_DEFAULT:
+        default:
+            ok = FALSE;
+            break;
+    }
+
+    return ok;
+}
+
+static gboolean
+pressure_value (gdouble inHg, PressureUnit to_unit, gdouble *value, PressureUnit def_unit)
+{
+    gboolean ok = TRUE;
+
+    *value = -1.0;
+
+    if (inHg < 0.0)
+	return FALSE;
+
+    if (to_unit == PRESSURE_UNIT_DEFAULT)
+	    to_unit = def_unit;
+
+    switch (to_unit) {
+        case PRESSURE_UNIT_INCH_HG:
+            *value = inHg;
+	    break;
+        case PRESSURE_UNIT_MM_HG:
+            *value = PRESSURE_INCH_TO_MM (inHg);
+	    break;
+        case PRESSURE_UNIT_KPA:
+            *value = PRESSURE_INCH_TO_KPA (inHg);
+	    break;
+        case PRESSURE_UNIT_HPA:
+            *value = PRESSURE_INCH_TO_HPA (inHg);
+	    break;
+        case PRESSURE_UNIT_MB:
+            *value = PRESSURE_INCH_TO_MB (inHg);
+	    break;
+        case PRESSURE_UNIT_ATM:
+            *value = PRESSURE_INCH_TO_ATM (inHg);
+	    break;
+        case PRESSURE_UNIT_INVALID:
+        case PRESSURE_UNIT_DEFAULT:
+        default:
+	    ok = FALSE;
+	    break;
+    }
+
+    return ok;
+}
+
+static gboolean
+distance_value (gdouble miles, DistanceUnit to_unit, gdouble *value, DistanceUnit def_unit)
+{
+    gboolean ok = TRUE;
+
+    *value = -1.0;
+
+    if (miles < 0.0)
+	return FALSE;
+
+    if (to_unit == DISTANCE_UNIT_DEFAULT)
+	    to_unit = def_unit;
+
+    switch (to_unit) {
+        case DISTANCE_UNIT_MILES:
+            *value = miles;
+            break;
+        case DISTANCE_UNIT_KM:
+            *value = VISIBILITY_SM_TO_KM (miles);
+            break;
+        case DISTANCE_UNIT_METERS:
+            *value = VISIBILITY_SM_TO_M (miles);
+            break;
+        case DISTANCE_UNIT_INVALID:
+        case DISTANCE_UNIT_DEFAULT:
+        default:
+	    ok = FALSE;
+	    break;
+    }
+
+    return ok;
+}
+
+gboolean
+weather_info_get_value_sky (WeatherInfo *info, WeatherSky *sky)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (sky != NULL, FALSE);
+
+    if (!info->valid)
+	return FALSE;
+
+    if (info->sky <= SKY_INVALID || info->sky >= SKY_LAST)
+	return FALSE;
+
+    *sky = info->sky;
+
+    return TRUE;
+}
+
+gboolean
+weather_info_get_value_conditions (WeatherInfo *info, WeatherConditionPhenomenon *phenomenon, WeatherConditionQualifier *qualifier)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (phenomenon != NULL, FALSE);
+    g_return_val_if_fail (qualifier != NULL, FALSE);
+
+    if (!info->valid)
+	return FALSE;
+
+    if (!info->cond.significant)
+	return FALSE;
+
+    if (!(info->cond.phenomenon > PHENOMENON_INVALID &&
+	  info->cond.phenomenon < PHENOMENON_LAST &&
+	  info->cond.qualifier > QUALIFIER_INVALID &&
+	  info->cond.qualifier < QUALIFIER_LAST))
+        return FALSE;
+
+    *phenomenon = info->cond.phenomenon;
+    *qualifier = info->cond.qualifier;
+
+    return TRUE;
+}
+
+gboolean
+weather_info_get_value_temp (WeatherInfo *info, TempUnit unit, gdouble *value)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (value != NULL, FALSE);
+
+    if (!info->valid)
+	return FALSE;
+
+    return temperature_value (info->temp, unit, value, info->temperature_unit);
+}
+
+gboolean
+weather_info_get_value_temp_min (WeatherInfo *info, TempUnit unit, gdouble *value)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (value != NULL, FALSE);
+
+    if (!info->valid || !info->tempMinMaxValid)
+	return FALSE;
+
+    return temperature_value (info->temp_min, unit, value, info->temperature_unit);
+}
+
+gboolean
+weather_info_get_value_temp_max (WeatherInfo *info, TempUnit unit, gdouble *value)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (value != NULL, FALSE);
+
+    if (!info->valid || !info->tempMinMaxValid)
+	return FALSE;
+
+    return temperature_value (info->temp_max, unit, value, info->temperature_unit);
+}
+
+gboolean
+weather_info_get_value_dew (WeatherInfo *info, TempUnit unit, gdouble *value)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (value != NULL, FALSE);
+
+    if (!info->valid)
+	return FALSE;
+
+    return temperature_value (info->dew, unit, value, info->temperature_unit);
+}
+
+gboolean
+weather_info_get_value_apparent (WeatherInfo *info, TempUnit unit, gdouble *value)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (value != NULL, FALSE);
+
+    if (!info->valid)
+	return FALSE;
+
+    return temperature_value (calc_apparent (info), unit, value, info->temperature_unit);
+}
+
+gboolean
+weather_info_get_value_update (WeatherInfo *info, time_t *value)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (value != NULL, FALSE);
+
+    if (!info->valid)
+	return FALSE;
+
+    *value = info->update;
+
+    return *value != 0;
+}
+
+gboolean
+weather_info_get_value_sunrise (WeatherInfo *info, time_t *value)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (value != NULL, FALSE);
+
+    if (!info->valid || !info->sunValid)
+	return FALSE;
+
+    *value = info->sunrise;
+
+    return *value != 0;
+}
+
+gboolean
+weather_info_get_value_sunset (WeatherInfo *info, time_t *value)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (value != NULL, FALSE);
+
+    if (!info->valid || !info->sunValid)
+	return FALSE;
+
+    *value = info->sunset;
+
+    return *value != 0;
+}
+
+gboolean
+weather_info_get_value_wind (WeatherInfo *info, SpeedUnit unit, gdouble *speed, WeatherWindDirection *direction)
+{
+    gboolean res = FALSE;
+
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (speed != NULL, FALSE);
+    g_return_val_if_fail (direction != NULL, FALSE);
+
+    if (!info->valid)
+	return FALSE;
+
+    if (info->windspeed < 0.0 || info->wind <= WIND_INVALID || info->wind >= WIND_LAST)
+        return FALSE;
+
+    res = speed_value (info->windspeed, unit, speed, info->speed_unit);
+    *direction = info->wind;
+
+    return res;
+}
+
+gboolean
+weather_info_get_value_pressure (WeatherInfo *info, PressureUnit unit, gdouble *value)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (value != NULL, FALSE);
+
+    if (!info->valid)
+	return FALSE;
+
+    return pressure_value (info->pressure, unit, value, info->pressure_unit);
+}
+
+gboolean
+weather_info_get_value_visibility (WeatherInfo *info, DistanceUnit unit, gdouble *value)
+{
+    g_return_val_if_fail (info != NULL, FALSE);
+    g_return_val_if_fail (value != NULL, FALSE);
+
+    if (!info->valid)
+	return FALSE;
+
+    return distance_value (info->visibility, unit, value, info->distance_unit);
+}
+
+static void
+_weather_internal_check (void)
+{
+    g_assert (G_N_ELEMENTS (wind_direction_str) == WIND_LAST);
+    g_assert (G_N_ELEMENTS (sky_str) == SKY_LAST);
+    g_assert (G_N_ELEMENTS (conditions_str) == PHENOMENON_LAST);
+    g_assert (G_N_ELEMENTS (conditions_str[0]) == QUALIFIER_LAST);
+}
